-- require('Scripts.Script_OL.utils')
-- require('Scripts.Script_OL.class')
----------------------------------------------------------------------------------------------------
local msg = function(M) reaper.ShowConsoleMsg(tostring(M).."\n") end
----------------------------------------------------------------------------------------------------

-- File

File = class(function(a, path)
  a.separator = "#####################"
  a.path = path
  a.data_s = Utils.readStringFromFile(path)
  a.tracks = {}
  a:parseTracks()
end)


function File:parseTracks()
  self.tracks = {}
  local chunks = Utils.explode(self.separator, self.data_s)
  for i=1, #chunks do
    local chunk = chunks[i]
    if chunk:len() > self.separator:len()+50 then
      table.insert(self.tracks, ChunkTrack(chunk.."#"))
    end
  end
end

function File:save()
  Utils.writeStringToFile(self.path, self:__tostring())
end

function File:findTrackById(id)
  for i=1,#self.tracks do
    local track = self.tracks[i]
    if track.id == id then
      return track
    end
  end
end

function File:__tostring()
  return Utils.join("", self.tracks) .. self.separator:sub(0, -2)
end

-- Track

ChunkTrack = class(function(a, chunk)
  -- msg(chunk)
  a.chunk = chunk
  a.id = ""
  a.header = ""
  a.footer = ""
  a.fxs = {}
  a:parseId()
  a:parseHeader()
  a:parseFooter()
  a:parseFXs()
end)

function ChunkTrack:parseId()
  self.id = string.match(self.chunk, "TRACKID ({[^}]+})")
end

function ChunkTrack:parseHeader()
  local j, i = string.find(self.chunk, "BYPASS")
  self.header = string.sub(self.chunk, 0, j-2)
end

function ChunkTrack:parseFooter()
  -- msg("~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
  -- msg(self.chunk)
  -- local j, i = string.find(self.chunk, ("WAK 0 0\n>\n>" or "WAK 0\n0\n>\n>"))
  -- local j, i = string.find(self.chunk, "WAK 0\s?\n?0\n>\n>")
  local j, i = self.chunk:find "WAK 0[%s\n]0\n>\n>"
  self.footer = string.sub(self.chunk, j+6)
end

function ChunkTrack:parseFXs()
  self.fxs = {}
  local fxs = self.chunk
  fxs = fxs:sub(string.len(self.header)+2)
  fxs = fxs:gsub(self.footer, "")
  for fxstring,v in fxs:gmatch("BYPASS.-WAK%s%d") do
    table.insert(self.fxs, FX(fxstring))
  end
end

function ChunkTrack:findFxById(id)
  if self.fxs ~= nil then
    for i=1,#self.fxs do
      local fx = self.fxs[i]
      if fx.id == id then
        return fx
      end
    end
  end
end

function ChunkTrack:addFx(fx)
  table.insert(self.fxs, fx)
end

function ChunkTrack:__tostring()
  return self.header .. "\n" .. Utils.join("\n", self.fxs) .. "\n" .. self.footer
end

-- FX

FX = class(function(a, body)
  a.body = body
  a.id = 0
  a:parseId()
end)

function FX:__tostring()
  return self.body
end

function FX:parseId()
  self.id = string.match(self.body, "FXID ({[^}]+})")
end
